"use strict";(self.webpackChunkjupytutor=self.webpackChunkjupytutor||[]).push([[312],{312:(t,e,s)=>{var l=s(318),n=s.n(l),r=s(879),a=s.n(r),u=s(237),A=s.n(u),o=s(862),i=s.n(o),c=s(50),b=s.n(c),d=s(495),m=s.n(d),y=s(965),h={};h.styleTagTransform=m(),h.setAttributes=i(),h.insert=A().bind(null,"head"),h.domAPI=a(),h.insertStyleElement=b(),n()(y.A,h),y.A&&y.A.locals&&y.A.locals;var p=s(594),f={};f.styleTagTransform=m(),f.setAttributes=i(),f.insert=A().bind(null,"head"),f.domAPI=a(),f.insertStyleElement=b(),n()(p.A,f),p.A&&p.A.locals&&p.A.locals}}]);